function [retval] = performance(Pt, Pdef, Pm, B, Nc, Nap, terrainX, R)
% Performance of a client-centric scheme
% Calculates and plots system performance (ratio of discovered graph edges)
% and evolution of mean reputations and scores for truthful users and
% attackers.
% Pt: Truthful probability
% Pdef: Defection probability for an attacker
% Pm: Probability an AP is managed
% B: discount factor
% Nc: #clients
% Nap: #APs
% terrainX: size of terrain
% R: range/radius
%
% Copyright (c) 2011, Pantelis A. Frangoudis
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:
%
%    * Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

    R1 = 0.0;
    R2 = 0.0;
    Rep = 0.0;
    rounds = 50;
    show_ci = 0;
    
    for i=1:rounds
        repsT(i) = R1;
        repsA(i) = R2;
        reps(i) = Rep;

        % Efficiency of a pure AP-centric scheme at round i
        toplotAC(i) = efficiency_ap_centric(Nc, Nap, Pm, terrainX, R);

        % Efficiency of a client-driven scheme at round i
        theround = efficiency(Nc, Nap, Pt, Pdef, Pm, R2, R1, terrainX, R);

        % an isolated user does not contribute reports and his reputation
        % is not discounted
        Pi = Pisolated(Nap, R, Pm, terrainX);

        % Score calculation for the two types of users
        Tscore = score(Nc, Nap, Pt, Pdef, Pm, R2, R1, 'T', terrainX, R);
        Ascore = score(Nc, Nap, Pt, Pdef, Pm, R2, R1, 'A', terrainX, R);

        % reputation of truthful users
        R1 = Pi .* R1 + (1.0 - Pi) .* (B * R1 + (1.0 - B) * Tscore);

        % reputation of attackers
        R2 = Pi .* R2 + (1.0 - Pi) .* (Pdef .* B .* R2 + (1 - Pdef) .* ( B .* R2 + (1 - B) .* Ascore));
               
        toplot(i) = theround;
        scoreA(i) = Ascore;
        scoreT(i) = Tscore;
    end

    % Plot simulation data with confidence intervals
    if show_ci == 1
        load file1.dat; X=file1(:,1); Y=file1(:,2); EY=file1(:,3); YRA=file1(:,4); EYRA=file1(:,5); YRT=file1(:,6); EYRT=file1(:,7);

        x=1:rounds;
        % plot detection probability
        ah = axes();
        set(ah, 'FontSize', 14);

        errorbar(X, Y, EY, 'r+');
        hold on
        plot(x, toplot, 'b+'); 
        hold on;
        plot(x, toplotAC);
        hold off
        xlabel('Round');
        ylabel('Ratio of detected edges');
        title({'Topology discovery accuracy';'(12000 users/km^2, 50% attackers, 0.8 attacker prob., 2123 APs/km^2, 7% managed APs)'});
        legend('Client-centric (simulation)', 'Client-centric (analysis)', 'AP-centric', 3, 'Location', 'SouthEast');
        axis ([0 rounds 0.0 1.0]);
        figure 

        % plot avg reputation of potential attackers
        ah = axes();
        set(ah, 'FontSize', 14);

        errorbar(X, YRA, EYRA, 'r+');
        hold on
        plot(x, repsA, 'b+');
        hold off
        xlabel('Round');
        ylabel('Reputation');        
        title({'Average attacker reputation';'(12000 users/km^2, 50% attackers, 0.8 attacker prob., 2123 APs/km^2, 7% managed APs)'});
        legend('Simulation', 'Analysis', 2, 'Location', 'NorthEast');
        axis ([0 rounds 0.0 1.0]);
%        print -dpng plots/AUEB-repsA.png

        figure 

        % plot avg reputation of truthful users
        ah = axes();
        set(ah, 'FontSize', 14);

        errorbar(X, YRT, EYRT, 'r+');
        hold on
        plot(x, repsT, 'b+');
        hold off
        xlabel('Round');
        ylabel('Reputation');        
        title({'Average truthful user reputation';'(12000 users/km^2, 50% attackers, 0.8 attacker prob., 2123 APs/km^2, 7% managed APs)'});
        legend('Simulation', 'Analysis', 2, 'Location', 'SouthEast');
%        print -dpng plots/AUEB-repsT.png
        axis ([0 rounds 0.0 1.0]);

    else
        % plot without confidence intervals [all in one figure]
        load file1.dat; X=file1(:,1); YDP=file1(:,2); YRA=file1(:,3); YSA=file1(:,4); YRT=file1(:,5); YST=file1(:,6);

        x=1:rounds;
        
        ah = axes();
        set(ah, 'FontSize', 22);

        plot(X, YDP, 'r+');
        hold on
        plot(x, toplot, 'bo'); 
        hold on;
        plot(x, toplotAC);
        hold on 
        plot(X, YRT, 'k^');
        hold on
        plot(x, repsT, 'ks');
        hold on
        plot(X, YRA, 'mV');
        hold on
        plot(x, repsA, 'mp');
        hold off    
 
        axis ([0 rounds 0.0 1.0]);
        title({'Filtering efficiency and evolution of reputations';'10000 clients/km^2, 2123 APs/km^2';'p_{t}=0.5, p_{a}=0.9, p_{m}=0.07'});
%        title('Average attacker score (p_{m}=0.07)');
        xlabel('Round');
        ylabel('Ratio of discovered edges, Reputation');
%        legend('Simulation', 'Analysis', 2, 'Location', 'SouthEast');
        leg = legend('User-centric (sim)', 'User-centric (analysis)', 'AP-centric', 'Honest rep. (sim)', 'Honest rep.(analysis)', 'Attacker rep. (sim)', 'Attacker rep. (analysis)', 7, 'Location', 'East');
        set(leg, 'FontSize', 13);
    end
end
%